#!/usr/bin/env python

# python documentation should be here (not yet)
# Usage: try -h option
#        This program read fifo overflow register on the NEUNET equipment(s)

import getopt, socket, struct, sys, time

def usage():
    usage_message = """\
Usage: lookup_fifo.py [-a] [-h] [-t sec] [-w] [-W ratio] ip_address [ip_address ...]
-a: Audible when print warning.
-h: Print usage and exit.
-t: Interval time (in seconds) between each lookup.  Default is 1 second.
-w: Print warning only.
-W: Specify the warning FIFO percentage.  For example, -W 50 means print
    warning if the number of events in FIFO exceeds 50% of the FIFO buffer
    size (32MB).  Default value is 80%.
Output Format:
Current_Time Seconds_from_Start IP_address FIFO_Overflow_Count FIFO_Event_Num
Sample Ouptupt:
14:47:30 64 192.168.0.17 4 32513 (kB)
"""
    sys.stderr.write(usage_message)

def print_time(start_time):
    current_time = time.time()
    print time.strftime('%H:%M:%S', time.localtime(current_time)),
    print int(current_time - start_time),

def print_fifo_info(start_time, ip_address, fifo_overflow_count, fifo_ev_kb):
    print ip_address, fifo_overflow_count, fifo_ev_kb, '(kB)'
    
########################################################################
# constants
########################################################################
tcp_port       = 23
udp_port       = 4660
fifo_address   = 0x9c
fifo_reg_len   = 4
bcp_header_len = 8
WRITE          = 0x80
READ           = 0xc0

########################################################################
# options default values
########################################################################
interval       = 1      # 1 sec
warning_only   = False
audible        = False
warning_percent  = 80

########################################################################
# main
########################################################################
try:
    opts, ip_addresses = getopt.getopt(sys.argv[1:], 'awW:t:')
except getopt.GetoptError:
    usage()
    sys.exit(1)

for opt, arg in opts:
    if opt == '-a':
        audible = True
    if opt == '-h':
        usage()
        sys.exit(0)
    if opt == '-w':
        warning_only = True
    if opt == '-W':
        warning_percent = int(arg)
    if opt == '-t':
        interval = int(arg)

if len(ip_addresses) == 0:
    usage()
    sys.exit(1)

u = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

start_time = time.time()

while True:
    output_lines = 0
    for ip_address in ip_addresses:
        seq_num   = 1
        send_data = struct.pack('>BBBBI', 0xff, READ, seq_num,
                        fifo_reg_len, fifo_address)
        u.sendto(send_data, (ip_address, udp_port))
        reply_packet = u.recvfrom(bcp_header_len + fifo_reg_len)
        # u.recvfrom()[0]: UDP payload
        # u.recvfrom()[1]: (host, address) tuple
        reply_data = struct.unpack('>BBBBIBBBB', reply_packet[0])
        fifo_of_count = int(reply_data[5])
        fifo_ev_num   =  (int(reply_data[6]) * 255 * 255
                        + int(reply_data[7]) * 255
                        + int(reply_data[8]))
        fifo_ev_kb    = fifo_ev_num * 4 / 1024 

        if warning_only == False:
            print_time(start_time)
            print_fifo_info(start_time, ip_address, fifo_of_count, fifo_ev_kb)
            output_lines += 1

        fifo_ev_percent = float(fifo_ev_kb) / float(32513) * 100.0
        if fifo_ev_percent > warning_percent:
            print_time(start_time)
            print 'WARNING: %s Event %% in FIFO: %5.1f %% Overflow Count: %d' % (ip_address, fifo_ev_percent, fifo_of_count),
            output_lines += 1
            if audible:
                print '\a\n',
            else:
                print '\n',

    if len(ip_addresses) > 1 and output_lines > 1:
            print '---'
    # XXX: How can I supress keyboard interrupt message when Ctrl-C pressed?
    #      This may not be right.
    try:
        time.sleep(interval)
    except KeyboardInterrupt:
        sys.exit(0)
